/*
 **************************************************************************
 *		SWI-Prolog ActiveX Extensions.
 *
 *	This software is provided in source and binary form free of charge to 
 *  users of SWI-Prolog.  No warranty of any kind is expressed or implied;
 *  in particular, this is entirely unsupported software.
 *
 *  Documentation for this product is available from the SWI-Prolog 
 *  web site (http://www.swi.psy.uva.nl/projects/SWI-Prolog/).
 *
 *	Version 1.0.1.
 *
 *	October 1, 1999
 *
 **************************************************************************
 */

//
//	swiactx.h
//
#pragma once
#include "swixcvt.h"

#ifdef SWIACTX_DLL
    //  We're building the DLL (exporting the class)
    #define SWIACTX_RESIDENT      __declspec(dllexport)
    #define SWIACTX_NON_RESIDENT  __declspec(dllimport)
#else
    //  We're using the DLL (importing the class)
    #define SWIACTX_RESIDENT      __declspec(dllimport)
    #define SWIACTX_NON_RESIDENT  __declspec(dllexport)
#endif


extern atom_t   		atomActxError;  		//  "activeXError"
//  Boolean atoms
extern atom_t   		atomTrue;       		//  'true'
extern atom_t   		atomFalse;      		//  'false'
//  Function invocation type atoms
extern atom_t   		atomFunc;       		//  'func'
extern atom_t   		atomPropGet;    		//  'propget'
extern atom_t   		atomPropPut;    		//  'propput'
extern atom_t   		atomPropPutRef; 		//  'propputref'
//  Function argument atoms
extern atom_t   		atomOptional;   		//  'optional'
//  Generated functors
extern functor_t		fnctrActxObject;		//  "activeXObject"
extern functor_t		fnctrActxError; 		//  "activeXError"
extern functor_t		fnctrActxSafearray;		//  "activeXSafearray"
extern functor_t		fnctrActxVariant;		//	"activeXVariant"

extern ACTXJOURNAL actxJournal;

///////////////////////////////////////////////////////////////////////
//      ActiveX/IDispatch parameter management
///////////////////////////////////////////////////////////////////////

typedef vector<_variant_t> VVARS;
typedef vector<DISPID> VDISPID;

class ACTXDPARAMS : public DISPPARAMS
{
  public:
	ACTXDPARAMS ( int nArgs = 0 )
	{
		init( nArgs );
	}

	VVARS & VVars ()
		{ return _vvars; }      
	VDISPID & Vdispid ()
		{ return _vdispid; }

	void PreparePropPut ( int iarg )
	{
		_vdispid.resize(1);
		_vdispid[0] = DISPID_PROPERTYPUT;
		rgdispidNamedArgs = & _vdispid[0];
		cNamedArgs = 1;

	}
	void resize ( int nArgs )
	{
		_vvars.clear();
		_vvars.resize( nArgs );
		rgvarg = & _vvars[0];
		cArgs = nArgs;
	}

  protected:
	VVARS _vvars;   		//  Dynamic array of variants
	VDISPID _vdispid;       //  Dynamic array of DISPIDs

  protected:
	void init ( int nArgs )
	{
		cArgs = 0;
		cNamedArgs = 0;
		rgvarg = 0;
		rgdispidNamedArgs = 0;
		resize( nArgs );
	}
};

///////////////////////////////////////////////////////////////////////
//      ActiveX/IDispatch exception management
///////////////////////////////////////////////////////////////////////
struct ACTXEXCEPINFO : public EXCEPINFO
{
	ACTXEXCEPINFO ()
	{
		wCode = 0;
		wReserved = 0;
		bstrSource = 0;
		bstrDescription = 0;
		bstrHelpFile = 0;
		dwHelpContext = 0;
		pvReserved = 0;
		pfnDeferredFillIn = 0;
		scode = 0;
	}
	~ ACTXEXCEPINFO ()
	{
		if ( bstrSource )
			::SysFreeString( bstrSource );
		if ( bstrDescription )
			::SysFreeString( bstrDescription );
		if ( bstrHelpFile )
			::SysFreeString( bstrHelpFile );
	}
};


//  Keyword enum for common atoms
enum EKEYWD
{
	ENONE,
	ETRUE,
	EFALSE,
	EOPTIONAL,
	EMAX
};


//  Information necessary to continue backtracking
struct BKTRKENUM
{
	//  Enumeration variant 
	IEnumVARIANTPtr _ipEnum;
	//  Token for any returned interface pointer
	ACTXTOKEN _atkn;
	//  Debugging
	bool _bActive;


	BKTRKENUM ()
		: _atkn(-1),
		_bActive(true)
	{
		_atkn = actxJournal.nextToken();
	}

	~ BKTRKENUM ()
	{
		assert( this );
		actxJournal.erase( _atkn );
		_ipEnum = NULL;
		assert( _bActive );
		_bActive = false;
	}
};


struct BKTRKCONTEXT
{       
	int _iContext;

	BKTRKCONTEXT ()
	{
		_iContext = actxJournal.IPushContext();
	}
	~ BKTRKCONTEXT ()
	{
		actxJournal.PopContext( _iContext );
	}
};



extern "C"
{
	SWIACTX_RESIDENT ACTXJOURNAL * PSwiActxGetJournal ();
};

///////////////////////////////////////////////////////////////////////
//      Forward declarations
///////////////////////////////////////////////////////////////////////
//  Generic functions
void init_constants();
EKEYWD eKeywordCheck ( term_t t );
DWORD invokeAtom ( term_t tKind );
foreign_t raisePrologException ( term_t tExcept );
bool bHrFail ( HRESULT hr );
HRESULT hrCreateInstance ( IP_IUnknown & ipUnk, 
							TSZC tszcProgIdOrClassId, 
							DWORD dwClsContext = CLSCTX_ALL );

//  Validation and conversion functions

HRESULT hrConvertArgList ( IP_ITypeInfo & ipInfo,
							BINDPTR & bindPtr,
							term_t tArgList,
							ACTXDPARAMS & params, 
							ZSTR & zsError );

HRESULT hrGetInvokeInfo ( 
	IP_IDispatch & ipId,    		//  [in]	IDispatch interface ref
	TSZC tszcMethod,				//  [in]	Name of method
	DWORD invkind,  				//  [in]	Type of invocation (INVOKEKIND)
	MEMBERID & mbrId,       		//  [out]   Member id
	IP_ITypeInfo & ipInfo,  		//  [out]   Interface type info
	BINDPTR & bindPtr );     		//  [out]   Function binding information

HRESULT hrGetInvokeInfo ( 
	IP_IDispatch & ipId,    		//  [in]	IDispatch interface ref
	TSZC tszcMethod,				//  [in]	Name of method
	DWORD invkind,  				//  [in]	Type of invocation
	MEMBERID & mbrId );      		//  [out]	Member id

bool bParseInvocationList ( 
	term_t tInvokeList,     		//  [in]    list term
	DWORD & dwInvoke,       		//  [out]   OLE invocation flags
	TSZC & tszcMethod );     		//  [out]   Method name string

HRESULT hrCollectionCount ( IP_IDispatch & ipId, int & iCount );

//  Foreign predicates
foreign_t actxCreateObject ( term_t tProgId, term_t tIDispatchFunctor );
foreign_t actxReleaseObject ( term_t tIDispatchFunctor );
foreign_t actxQueryInterface ( term_t tIDispatchFunctor, term_t tGuid, term_t tResult );
foreign_t actxInvokeObject ( term_t tIDispatchFunctor, 
							term_t tInvokeList,
							term_t tArgList,
							term_t tResult );

foreign_t actxEnumObject ( term_t tIDispatchFunctor, 
							term_t tResult,
							foreign_t fHand );
foreign_t actxCollectionList ( term_t tIDispatchFunctor, term_t tResult );
foreign_t actxCloneObject ( term_t tIDispatchFunctor, term_t tResult );
foreign_t actxReleaseAll ();
foreign_t actxBreakCheck();
foreign_t actxContext ( foreign_t fHand );
foreign_t actxContextGlobal ( term_t tIDispatchFunctor, term_t tBool );
foreign_t actxListToDate ( term_t tList, term_t tDate );


//  End of SWIACTX.H
